/*******************************************************************************
 * @file    ADC.c
 * @author  King
 * @version V1.00
 * @date    27-Nov-2021
 * @brief   ......
*******************************************************************************/


/* Define to prevent recursive inclusion -------------------------------------*/
#define __ADC_C__


/* Includes ------------------------------------------------------------------*/
#include "ADC.h"


#if ADC_ENABLE


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/


/* Exported variables --------------------------------------------------------*/
/* Exported function prototypes ----------------------------------------------*/


/*******************************************************************************
 * @brief       
 * @param       
 * @retval      
 * @attention   
*******************************************************************************/
void ADC_Configure(void)
{
    FL_GPIO_InitTypeDef      GPIO_InitStruct;
    FL_ADC_CommonInitTypeDef ADC_CommonInitStruct;
    FL_ADC_InitTypeDef       ADC_InitStruct ;

    FL_GPIO_StructInit(&GPIO_InitStruct);
    GPIO_InitStruct.pin        = FL_GPIO_PIN_10 | FL_GPIO_PIN_11 | FL_GPIO_PIN_12;
    GPIO_InitStruct.mode       = FL_GPIO_MODE_ANALOG;
    GPIO_InitStruct.outputType = FL_GPIO_OUTPUT_PUSHPULL;
    FL_GPIO_Init(GPIOC, &GPIO_InitStruct);

    FL_ADC_CommonStructInit(&ADC_CommonInitStruct);
    ADC_CommonInitStruct.operationSource   = FL_ADC_CLK_SOURCE_ADCCLK;
    ADC_CommonInitStruct.clockSource       = FL_CMU_ADC_CLK_SOURCE_RCHF;
    ADC_CommonInitStruct.clockPrescaler    = FL_CMU_ADC_PSC_DIV8;
    ADC_CommonInitStruct.APBClockPrescaler = FL_ADC_APBCLK_PSC_DIV1;
    ADC_CommonInitStruct.referenceSource   = FL_ADC_REF_SOURCE_VDDA;
    ADC_CommonInitStruct.bitWidth          = FL_ADC_BIT_WIDTH_12B;
    FL_ADC_CommonInit(&ADC_CommonInitStruct);

    FL_ADC_StructInit(&ADC_InitStruct);
    ADC_InitStruct.conversionMode        = FL_ADC_CONV_MODE_SINGLE;
    ADC_InitStruct.autoMode              = FL_ADC_SINGLE_CONV_MODE_AUTO;
    ADC_InitStruct.waitMode              = FL_ENABLE;
    ADC_InitStruct.overrunMode           = FL_ENABLE;
    ADC_InitStruct.scanDirection         = FL_ADC_SEQ_SCAN_DIR_FORWARD;
    ADC_InitStruct.externalTrigConv      = FL_ADC_TRIGGER_EDGE_NONE;
    ADC_InitStruct.triggerSource         = FL_ADC_TRGI_LUT0;
    ADC_InitStruct.fastChannelTime       = FL_ADC_FAST_CH_SAMPLING_TIME_2_ADCCLK;
    ADC_InitStruct.lowChannelTime        = FL_ADC_SLOW_CH_SAMPLING_TIME_192_ADCCLK;
    ADC_InitStruct.oversamplingMode      = FL_ENABLE;
    ADC_InitStruct.overSampingMultiplier = FL_ADC_OVERSAMPLING_MUL_8X;
    ADC_InitStruct.oversamplingShift     = FL_ADC_OVERSAMPLING_SHIFT_3B;
    FL_ADC_Init(ADC, &ADC_InitStruct);

    TASK_Append(TASK_ID_ADC, ADC_Handler, 500);
}


/*******************************************************************************
 * @brief       
 * @param       
 * @retval      
 * @attention   
*******************************************************************************/
uint32_t ADC_GetSingleChannelSample(uint32_t Channel)
{
    uint16_t Data = 0;

    FL_CMU_SetADCPrescaler(FL_CMU_ADC_PSC_DIV1);
    FL_ADC_EnableSequencerChannel(ADC, Channel);

    FL_ADC_ClearFlag_EndOfConversion(ADC);
    FL_ADC_Enable(ADC);

    FL_ADC_EnableSWConversion(ADC);
    while(FL_ADC_IsActiveFlag_EndOfConversion(ADC) == FL_RESET);

    FL_ADC_ClearFlag_EndOfConversion(ADC);
    Data = FL_ADC_ReadConversionData(ADC);

    FL_ADC_Disable(ADC);
    FL_ADC_DisableSequencerChannel(ADC, Channel);

    return Data;
}


/*******************************************************************************
 * @brief       
 * @param       
 * @retval      
 * @attention   
*******************************************************************************/
void ADC_Handler(void)
{
    uint32_t CH15_Data = 0, CH16_Data = 0, CH17_Data = 0;

    CH15_Data = ADC_GetSingleChannelSample(FL_ADC_EXTERNAL_CH15);
    CH16_Data = ADC_GetSingleChannelSample(FL_ADC_EXTERNAL_CH16);
    CH17_Data = ADC_GetSingleChannelSample(FL_ADC_EXTERNAL_CH17);

    printf("\r\nCH15 : %0.1fV, CH16 : %0.1fV, CH17 : %0.1fV",
            (float)CH15_Data / 4095 * 3.3,
            (float)CH16_Data / 4095 * 3.3,
            (float)CH17_Data / 4095 * 3.3);
}


#endif


/******************* (C) COPYRIGHT 2021 *************************END OF FILE***/

