/*******************************************************************************
 * @file    LCD.c
 * @author  King
 * @version V1.00
 * @date    27-Nov-2021
 * @brief   ......
*******************************************************************************/


/* Define to prevent recursive inclusion -------------------------------------*/
#define __LCD_C__


/* Includes ------------------------------------------------------------------*/
#include "LCD.h"


#if LCD_ENABLE


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/


/* Private variables ---------------------------------------------------------*/
uint32_t LCD_DisplayBuffer[10];


/* Private variables ---------------------------------------------------------*/
const DIGITRON_STRUCT DIGITRON_TABLE[38] = 
{
    {' ', {0, 0, 0, 0, 0, 0, 0, 0}},
    {'0', {1, 1, 1, 1, 1, 1, 0, 0}},
    {'1', {0, 1, 1, 0, 0, 0, 0, 0}},
    {'2', {1, 1, 0, 1, 1, 0, 1, 0}},
    {'3', {1, 1, 1, 1, 0, 0, 1, 0}},
    {'4', {0, 1, 1, 0, 0, 1, 1, 0}},
    {'5', {1, 0, 1, 1, 0, 1, 1, 0}},
    {'6', {1, 0, 1, 1, 1, 1, 1, 0}},
    {'7', {1, 1, 1, 0, 0, 0, 0, 0}},
    {'8', {1, 1, 1, 1, 1, 1, 1, 0}},
    {'9', {1, 1, 1, 1, 0, 1, 1, 0}},
    {'A', {1, 1, 1, 0, 1, 1, 1, 0}},
    {'b', {0, 0, 1, 1, 1, 1, 1, 0}},
    {'c', {0, 0, 0, 1, 1, 0, 1, 0}},
    {'C', {1, 0, 0, 1, 1, 1, 0, 0}},
    {'d', {0, 1, 1, 1, 1, 0, 1, 0}},
    {'E', {1, 0, 0, 1, 1, 1, 1, 0}},
    {'F', {1, 0, 0, 0, 1, 1, 1, 0}},
    {'g', {1, 1, 1, 1, 0, 1, 1, 0}},
    {'H', {0, 1, 1, 0, 1, 1, 1, 0}},
    {'h', {0, 0, 1, 0, 1, 1, 1, 0}},
    {'i', {0, 0, 1, 0, 0, 0, 0, 0}},
    {'I', {0, 0, 0, 0, 1, 1, 0, 0}},
    {'J', {0, 1, 1, 1, 1, 0, 0, 0}},
    {'l', {0, 0, 0, 0, 1, 1, 0, 0}},
    {'L', {0, 0, 0, 1, 1, 1, 0, 0}},
    {'n', {0, 0, 1, 0, 1, 0, 1, 0}},
    {'o', {0, 0, 1, 1, 1, 0, 1, 0}},
    {'O', {1, 1, 1, 1, 1, 1, 0, 0}},
    {'P', {1, 1, 0, 0, 1, 1, 1, 0}},
    {'q', {1, 1, 1, 0, 0, 1, 1, 0}},
    {'r', {0, 0, 0, 0, 1, 0, 1, 0}},
    {'S', {1, 0, 1, 1, 0, 1, 1, 0}},
    {'t', {0, 0, 0, 1, 1, 1, 1, 0}},
    {'u', {0, 0, 1, 1, 1, 0, 0, 0}},
    {'U', {0, 1, 1, 1, 1, 1, 0, 0}},
    {'y', {0, 1, 1, 1, 0, 1, 1, 0}},
    {'-', {0, 0, 0, 0, 0, 0, 1, 0}},
};


/* Private variables ---------------------------------------------------------*/
const char LCD_CS_TABLE[4][44][3] = 
{
/*SEGX    00   01   02   03   04   05   06   07   08   09   10   11   12   13   14   15   16   17   18   19   20   21   22   23   24   25   26   27   28   29   30   31   32   33   34   35   36   37   38   39   40   41   42   43*/
/*COM0*/{"1D","P1","2D","P2","3D","P3","4D","C1","C2","W5","L1","5F","5A","  ","  ","  ","  ","  ","  ","  ","  ","  ","  ","  ","9F","9A","AF","AA","  ","  ","  ","  ","  ","  ","  ","  ","6F","6A","7F","7A","S4","S5","8F","8A"},
/*COM1*/{"1E","1C","2E","2C","3E","3C","4E","4C","C3","W4","L2","5G","5B","  ","  ","  ","  ","  ","  ","  ","  ","  ","  ","  ","9G","9B","AG","AB","  ","  ","  ","  ","  ","  ","  ","  ","6G","6B","7G","7B","S3","S6","8G","8B"},
/*COM2*/{"1G","1B","2G","2B","3G","3B","4G","4B","T1","W3","L3","5E","5C","  ","  ","  ","  ","  ","  ","  ","  ","  ","  ","  ","9E","9C","AE","AC","  ","  ","  ","  ","  ","  ","  ","  ","6E","6C","7E","7C","S2","S7","8E","8C"},
/*COM3*/{"1F","1A","2F","2A","3F","3A","4F","4A","W1","W2","L4","5D","P5","  ","  ","  ","  ","  ","  ","  ","  ","  ","  ","  ","9D","P9","AD","S9","  ","  ","  ","  ","  ","  ","  ","  ","6D","P6","7D","P7","S1","S8","8D","P8"},
};


/* Private variables ---------------------------------------------------------*/
const char DISPLAY_DIGIT_TABLE[10][7][3] = 
{
    {"1A", "1B", "1C", "1D", "1E", "1F", "1G"},
    {"2A", "2B", "2C", "2D", "2E", "2F", "2G"},
    {"3A", "3B", "3C", "3D", "3E", "3F", "3G"},
    {"4A", "4B", "4C", "4D", "4E", "4F", "4G"},
    {"5A", "5B", "5C", "5D", "5E", "5F", "5G"},
    {"6A", "6B", "6C", "6D", "6E", "6F", "6G"},
    {"7A", "7B", "7C", "7D", "7E", "7F", "7G"},
    {"8A", "8B", "8C", "8D", "8E", "8F", "8G"},
    {"9A", "9B", "9C", "9D", "9E", "9F", "9G"},
    {"AA", "AB", "AC", "AD", "AE", "AF", "AG"},
};


const char DISPLAY_POINT_TABLE[11][3] = {"P1", "P2", "P3", "P5", "P6", "P7", "P8", "P9", "C1", "C2", "C3"};
const char DISPLAY_BAT_TABLE[5][3]    = {"W1", "W2", "W3", "W4", "W5"};
const char DISPLAY_CSQ_TABLE[4][3]    = {"L1", "L2", "L3", "L4"};
const char DISPLAY_UNIT_TABLE[10][3]  = {"S1", "S2", "S3", "S4", "S5", "S6", "S7", "S8", "S9", "T1"};


/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/


/* Exported variables --------------------------------------------------------*/
/* Exported function prototypes ----------------------------------------------*/


/*******************************************************************************
 * @brief       
 * @param       
 * @retval      
 * @attention   
*******************************************************************************/
void LCD_DisplayRefresh(void)
{
    FL_LCD_WriteData(LCD, FL_LCD_DATA_REG0, LCD_DisplayBuffer[0]);
    FL_LCD_WriteData(LCD, FL_LCD_DATA_REG1, LCD_DisplayBuffer[1]);
    FL_LCD_WriteData(LCD, FL_LCD_DATA_REG2, LCD_DisplayBuffer[2]);
    FL_LCD_WriteData(LCD, FL_LCD_DATA_REG3, LCD_DisplayBuffer[3]);
    FL_LCD_WriteData(LCD, FL_LCD_DATA_REG4, LCD_DisplayBuffer[4]);
    FL_LCD_WriteData(LCD, FL_LCD_DATA_REG5, LCD_DisplayBuffer[5]);
    FL_LCD_WriteData(LCD, FL_LCD_DATA_REG6, LCD_DisplayBuffer[6]);
    FL_LCD_WriteData(LCD, FL_LCD_DATA_REG7, LCD_DisplayBuffer[7]);
    FL_LCD_WriteData(LCD, FL_LCD_DATA_REG8, LCD_DisplayBuffer[8]);
    FL_LCD_WriteData(LCD, FL_LCD_DATA_REG9, LCD_DisplayBuffer[9]);
}


/*******************************************************************************
 * @brief       
 * @param       
 * @retval      
 * @attention   
*******************************************************************************/
void LCD_Init(void)
{
    FL_GPIO_InitTypeDef GPIO_InitStruct;
    FL_LCD_InitTypeDef  LCD_InitStruct;

    memset(LCD_DisplayBuffer, 0, sizeof(LCD_DisplayBuffer));

    FL_GPIO_StructInit(&GPIO_InitStruct);
    GPIO_InitStruct.pin        = FL_GPIO_PIN_0 | FL_GPIO_PIN_1 | FL_GPIO_PIN_2 |
                                 FL_GPIO_PIN_3 | FL_GPIO_PIN_8 | FL_GPIO_PIN_9 |
                                 FL_GPIO_PIN_10;
    GPIO_InitStruct.mode       = FL_GPIO_MODE_ANALOG;
    GPIO_InitStruct.outputType = FL_GPIO_OUTPUT_PUSHPULL;
    FL_GPIO_Init(GPIOA, &GPIO_InitStruct);


    FL_GPIO_StructInit(&GPIO_InitStruct);
    GPIO_InitStruct.pin        = FL_GPIO_PIN_4  | FL_GPIO_PIN_5  | FL_GPIO_PIN_6  |
                                 FL_GPIO_PIN_7  | FL_GPIO_PIN_8  | FL_GPIO_PIN_9  |
                                 FL_GPIO_PIN_10 | FL_GPIO_PIN_11 | FL_GPIO_PIN_13 |
                                 FL_GPIO_PIN_14;
    GPIO_InitStruct.mode       = FL_GPIO_MODE_ANALOG;
    GPIO_InitStruct.outputType = FL_GPIO_OUTPUT_PUSHPULL;
    FL_GPIO_Init(GPIOB, &GPIO_InitStruct);


    FL_GPIO_StructInit(&GPIO_InitStruct);
    GPIO_InitStruct.pin        = FL_GPIO_PIN_13 | FL_GPIO_PIN_14 | FL_GPIO_PIN_15;
    GPIO_InitStruct.mode       = FL_GPIO_MODE_ANALOG;
    GPIO_InitStruct.outputType = FL_GPIO_OUTPUT_PUSHPULL;
    FL_GPIO_Init(GPIOC, &GPIO_InitStruct);


    FL_GPIO_StructInit(&GPIO_InitStruct);
    GPIO_InitStruct.pin        = FL_GPIO_PIN_2 | FL_GPIO_PIN_3 | FL_GPIO_PIN_4 |
                                 FL_GPIO_PIN_5;
    GPIO_InitStruct.mode       = FL_GPIO_MODE_ANALOG;
    GPIO_InitStruct.outputType = FL_GPIO_OUTPUT_PUSHPULL;
    FL_GPIO_Init(GPIOD, &GPIO_InitStruct);


    FL_GPIO_StructInit(&GPIO_InitStruct);
    GPIO_InitStruct.pin        = FL_GPIO_PIN_0 | FL_GPIO_PIN_1 | FL_GPIO_PIN_2 |
                                 FL_GPIO_PIN_3 | FL_GPIO_PIN_5;
    GPIO_InitStruct.mode       = FL_GPIO_MODE_ANALOG;
    GPIO_InitStruct.outputType = FL_GPIO_OUTPUT_PUSHPULL;
    FL_GPIO_Init(GPIOE, &GPIO_InitStruct);


    FL_LCD_StructInit(&LCD_InitStruct);
    LCD_InitStruct.biasCurrent = FL_LCD_BIAS_CURRENT_HIGH;
    LCD_InitStruct.biasMode    = FL_LCD_BIAS_MODE_3BIAS;
    LCD_InitStruct.biasVoltage = FL_LCD_BIAS_VOLTAGE_LEVEL7;
    LCD_InitStruct.COMxNum     = FL_LCD_COM_NUM_4COM;
    LCD_InitStruct.waveform    = FL_LCD_WAVEFORM_TYPEA;
    LCD_InitStruct.displayFreq = 64;
    LCD_InitStruct.mode        = FL_LCD_DRIVER_MODE_INNER_RESISTER;
    FL_LCD_Init(LCD, &LCD_InitStruct);


    FL_LCD_EnableCOMEN(LCD, FL_LCD_COMEN_COM0);
    FL_LCD_EnableCOMEN(LCD, FL_LCD_COMEN_COM1);
    FL_LCD_EnableCOMEN(LCD, FL_LCD_COMEN_COM2);
    FL_LCD_EnableCOMEN(LCD, FL_LCD_COMEN_COM3);

    FL_LCD_EnableSEGEN0(LCD, FL_LCD_SEGEN0_SEG0);
    FL_LCD_EnableSEGEN0(LCD, FL_LCD_SEGEN0_SEG1);
    FL_LCD_EnableSEGEN0(LCD, FL_LCD_SEGEN0_SEG2);
    FL_LCD_EnableSEGEN0(LCD, FL_LCD_SEGEN0_SEG3);
    FL_LCD_EnableSEGEN0(LCD, FL_LCD_SEGEN0_SEG4);
    FL_LCD_EnableSEGEN0(LCD, FL_LCD_SEGEN0_SEG5);
    FL_LCD_EnableSEGEN0(LCD, FL_LCD_SEGEN0_SEG6);
    FL_LCD_EnableSEGEN0(LCD, FL_LCD_SEGEN0_SEG7);
    FL_LCD_EnableSEGEN0(LCD, FL_LCD_SEGEN0_SEG8);
    FL_LCD_EnableSEGEN0(LCD, FL_LCD_SEGEN0_SEG9);
    FL_LCD_EnableSEGEN0(LCD, FL_LCD_SEGEN0_SEG10);
    FL_LCD_EnableSEGEN0(LCD, FL_LCD_SEGEN0_SEG11);
    FL_LCD_EnableSEGEN0(LCD, FL_LCD_SEGEN0_SEG12);

    FL_LCD_EnableSEGEN0(LCD, FL_LCD_SEGEN0_SEG24);
    FL_LCD_EnableSEGEN0(LCD, FL_LCD_SEGEN0_SEG25);
    FL_LCD_EnableSEGEN0(LCD, FL_LCD_SEGEN0_SEG26);
    FL_LCD_EnableSEGEN0(LCD, FL_LCD_SEGEN0_SEG27);

    FL_LCD_EnableSEGEN1(LCD, FL_LCD_SEGEN1_SEG4);   //SEG36
    FL_LCD_EnableSEGEN1(LCD, FL_LCD_SEGEN1_SEG5);   //SEG37
    FL_LCD_EnableSEGEN1(LCD, FL_LCD_SEGEN1_SEG6);   //SEG38
    FL_LCD_EnableSEGEN1(LCD, FL_LCD_SEGEN1_SEG7);   //SEG39
    FL_LCD_EnableSEGEN1(LCD, FL_LCD_SEGEN1_SEG8);   //SEG40
    FL_LCD_EnableSEGEN1(LCD, FL_LCD_SEGEN1_SEG9);   //SEG41
    FL_LCD_EnableSEGEN1(LCD, FL_LCD_SEGEN1_SEG10);  //SEG42
    FL_LCD_EnableSEGEN1(LCD, FL_LCD_SEGEN1_SEG11);  //SEG43

    FL_LCD_Enable(LCD);

    LCD_DisplayRefresh();

    TASK_Append(TASK_ID_LCD, LCD_DisplayHandler, 100);
}


/*******************************************************************************
 * @brief       
 * @param       
 * @retval      
 * @attention   
*******************************************************************************/
void LCD_SearchCS(const char *str, uint8_t *com, uint8_t *seg)
{
    for(uint8_t i = 0; i < 4; i++)          //ɨ04COM
    {
        for(uint8_t j = 0; j < 44; j++)     //ɨ44SEG
        {
            if(strcmp(str, LCD_CS_TABLE[i][j]) == 0)
            {
                *com = i; *seg = j; return;
            }
        }
    }

    *com = 0xFF;
    *seg = 0xFF;
}


/*******************************************************************************
 * @brief       
 * @param       
 * @retval      
 * @attention   
*******************************************************************************/
void LCD_DisplayDigit(uint8_t Index, char ch)
{
    uint8_t i    = 0, j   = 0;
    uint8_t com  = 0, seg = 0;

    for(i = 0; i < 38; i++)
    {
        if(DIGITRON_TABLE[i].ch == ch)
        {
            j = i; break;
        }
    }

    if(i == 38)
    {
        printf("\r\nUnknown Character"); return;
    }

    for(i = 0; i < 7; i++)
    {
        LCD_SearchCS(DISPLAY_DIGIT_TABLE[Index][i], &com, &seg);

        if((com != 0xFF) && (seg != 0xFF))
        {
            FL_LCD_4COMDisplay(LCD_DisplayBuffer, com, seg, DIGITRON_TABLE[j].segment[i]);
        }
    }

    LCD_DisplayRefresh();
}


#define POINT_DP1   0x0001
#define POINT_DP2   0x0002
#define POINT_DP3   0x0004
#define POINT_DP5   0x0008
#define POINT_DP6   0x0010
#define POINT_DP7   0x0020
#define POINT_DP8   0x0040
#define POINT_DP9   0x0080
#define POINT_COL1  0x0100
#define POINT_COL2  0x0200
#define POINT_COL3  0x0400


/*******************************************************************************
 * @brief       
 * @param       
 * @retval      
 * @attention   
*******************************************************************************/
void LCD_DisplayPoint(uint16_t Mask)
{
    uint8_t com = 0, seg = 0;

    uint8_t Number = sizeof(DISPLAY_POINT_TABLE) / sizeof(DISPLAY_POINT_TABLE[0]);

    for(uint8_t i = 0; i < Number; i++)
    {
        LCD_SearchCS(DISPLAY_POINT_TABLE[i], &com, &seg);

        if((com != 0xFF) && (seg != 0xFF))
        {
            if((0x0001 << i) & Mask)
            {
                FL_LCD_4COMDisplay(LCD_DisplayBuffer, com, seg, 1);
            }
            else
            {
                FL_LCD_4COMDisplay(LCD_DisplayBuffer, com, seg, 0);
            }
        }
    }

    LCD_DisplayRefresh();
}


/*******************************************************************************
 * @brief       
 * @param       
 * @retval      
 * @attention   
*******************************************************************************/
void LCD_DisplayBAT(uint16_t Mask)
{
    uint8_t com = 0, seg = 0;

    uint8_t Number = sizeof(DISPLAY_BAT_TABLE) / sizeof(DISPLAY_BAT_TABLE[0]);

    for(uint8_t i = 0; i < Number; i++)
    {
        LCD_SearchCS(DISPLAY_BAT_TABLE[i], &com, &seg);

        if((com != 0xFF) && (seg != 0xFF))
        {
            if((0x0001 << i) & Mask)
            {
                FL_LCD_4COMDisplay(LCD_DisplayBuffer, com, seg, 1);
            }
            else
            {
                FL_LCD_4COMDisplay(LCD_DisplayBuffer, com, seg, 0);
            }
        }
    }

    LCD_DisplayRefresh();
}


/*******************************************************************************
 * @brief       
 * @param       
 * @retval      
 * @attention   
*******************************************************************************/
void LCD_DisplayCSQ(uint16_t Mask)
{
    uint8_t com = 0, seg = 0;

    uint8_t Number = sizeof(DISPLAY_CSQ_TABLE) / sizeof(DISPLAY_CSQ_TABLE[0]);

    for(uint8_t i = 0; i < Number; i++)
    {
        LCD_SearchCS(DISPLAY_CSQ_TABLE[i], &com, &seg);

        if((com != 0xFF) && (seg != 0xFF))
        {
            if((0x0001 << i) & Mask)
            {
                FL_LCD_4COMDisplay(LCD_DisplayBuffer, com, seg, 1);
            }
            else
            {
                FL_LCD_4COMDisplay(LCD_DisplayBuffer, com, seg, 0);
            }
        }
    }

    LCD_DisplayRefresh();
}


/*******************************************************************************
 * @brief       
 * @param       
 * @retval      
 * @attention   
*******************************************************************************/
void LCD_DisplayUnit(uint16_t Mask)
{
    uint8_t com = 0, seg = 0;

    uint8_t Number = sizeof(DISPLAY_UNIT_TABLE) / sizeof(DISPLAY_UNIT_TABLE[0]);

    for(uint8_t i = 0; i < Number; i++)
    {
        LCD_SearchCS(DISPLAY_UNIT_TABLE[i], &com, &seg);

        if((com != 0xFF) && (seg != 0xFF))
        {
            if((0x0001 << i) & Mask)
            {
                FL_LCD_4COMDisplay(LCD_DisplayBuffer, com, seg, 1);
            }
            else
            {
                FL_LCD_4COMDisplay(LCD_DisplayBuffer, com, seg, 0);
            }
        }
    }

    LCD_DisplayRefresh();
}


/*******************************************************************************
 * @brief       
 * @param       
 * @retval      
 * @attention   
*******************************************************************************/



/*******************************************************************************
 * @brief       
 * @param       
 * @retval      
 * @attention   
*******************************************************************************/
void LCD_DisplayClock(void)
{
    static uint8_t Hour = 15, Minute = 23, Second = 10;

    char DisplayString[10];

    memset(DisplayString, 0, sizeof(DisplayString));
    sprintf(DisplayString, "%02d%02d%02d", Hour, Minute, Second);

    for(uint8_t i = 0; i < strlen(DisplayString); i++)
    {
        LCD_DisplayDigit(4 + i, DisplayString[i]);
    }

    Second++;

    if(Second >= 60)
    {
        Second = 0;

        Minute++;

        if(Minute >= 60)
        {
            Minute = 0;

            Hour++;

            if(Hour >= 60)
            {
                Hour = 0;
            }
        }
    }
}


/*******************************************************************************
 * @brief       
 * @param       
 * @retval      
 * @attention   
*******************************************************************************/
void LCD_DisplayCount(void)
{
    static uint16_t Count = 0;

    char DisplayString[10];
    memset(DisplayString, 0, sizeof(DisplayString));

    sprintf(DisplayString, "%04d", Count++);

    for(uint8_t i = 0; i < strlen(DisplayString); i++)
    {
        LCD_DisplayDigit(i, DisplayString[i]);
    }
}


/*******************************************************************************
 * @brief       
 * @param       
 * @retval      
 * @attention   
*******************************************************************************/
void LCD_DisplayHandler(void)
{
    static uint32_t DisplayTick = 0;
    static uint16_t BAT_Index = 0, CSQ_Index = 0, Unit_Index = 0;
    static uint16_t COL_State = 0;

    LCD_DisplayCount();             //100ms

    if((DisplayTick % 2) == 0)      //200ms
    {
        switch(BAT_Index++ % 4)
        {
            case 0 : LCD_DisplayBAT(0x0001); break;
            case 1 : LCD_DisplayBAT(0x0001 | 0x0002); break;
            case 2 : LCD_DisplayBAT(0x0001 | 0x0002 | 0x0004); break;
            case 3 : LCD_DisplayBAT(0x0001 | 0x0002 | 0x0004 | 0x0008); break;
            default: break;
        }

        switch(CSQ_Index++ % 4)
        {
            case 0 : LCD_DisplayCSQ(0x0001); break;
            case 1 : LCD_DisplayCSQ(0x0001 | 0x0002); break;
            case 2 : LCD_DisplayCSQ(0x0001 | 0x0002 | 0x0004); break;
            case 3 : LCD_DisplayCSQ(0x0001 | 0x0002 | 0x0004 | 0x0008); break;
            default: break;
        }

        LCD_DisplayUnit(0x0001 << (Unit_Index++ % 10));
    }


    if((DisplayTick % 5) == 0)      //500ms
    {
        if(COL_State == 0)
        {
            COL_State = 1; LCD_DisplayPoint(POINT_COL2 | POINT_COL3);
        }
        else
        {
            COL_State = 0; LCD_DisplayPoint(0x0000);
        }
    }

    if((DisplayTick % 10) == 0)     //1000ms
    {
        LCD_DisplayClock();
    }

    DisplayTick += 1;
    DisplayTick %= 10000;
}


#endif


/******************* (C) COPYRIGHT 2021 *************************END OF FILE***/

