/*******************************************************************************
 * @file    LPTIMxx.c
 * @author  King
 * @version V1.00
 * @date    27-Nov-2021
 * @brief   ......
*******************************************************************************/


/* Define to prevent recursive inclusion -------------------------------------*/
#define __LPTIMxx_C__


/* Includes ------------------------------------------------------------------*/
#include "LPTIMxx.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/


/* Private variables ---------------------------------------------------------*/
volatile uint32_t LPTIM32_Tick = 0;


/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/


/* Exported variables --------------------------------------------------------*/
/* Exported function prototypes ----------------------------------------------*/


/*******************************************************************************
 * @brief       
 * @param       
 * @retval      
 * @attention   
*******************************************************************************/
void LPTIM32_Init(void)
{
    FL_LPTIM32_InitTypeDef LPTIM32_InitStruct;

    FL_LPTIM32_StructInit(&LPTIM32_InitStruct);
    LPTIM32_InitStruct.clockSource          = FL_CMU_LPTIM32_CLK_SOURCE_APBCLK;
    LPTIM32_InitStruct.mode                 = FL_LPTIM32_OPERATION_MODE_NORMAL;
    LPTIM32_InitStruct.prescalerClockSource = FL_LPTIM32_CLK_SOURCE_INTERNAL;
    LPTIM32_InitStruct.prescaler            = FL_LPTIM32_PSC_DIV8;
    LPTIM32_InitStruct.autoReload           = 1000 - 1;
    LPTIM32_InitStruct.onePulseMode         = FL_LPTIM32_ONE_PULSE_MODE_CONTINUOUS;
    LPTIM32_InitStruct.triggerEdge          = FL_LPTIM32_ETR_TRIGGER_EDGE_RISING;
    LPTIM32_InitStruct.countEdge            = FL_LPTIM32_ETR_COUNT_EDGE_RISING;
    FL_LPTIM32_Init(LPTIM32, &LPTIM32_InitStruct);

    FL_LPTIM32_ClearFlag_Update(LPTIM32);
    FL_LPTIM32_EnableIT_Update(LPTIM32);

    NVIC_DisableIRQ(LPTIMx_IRQn);
    NVIC_SetPriority(LPTIMx_IRQn, 2);
    NVIC_EnableIRQ(LPTIMx_IRQn);

    FL_LPTIM32_Enable(LPTIM32);
}


/*******************************************************************************
 * @brief       
 * @param       
 * @retval      
 * @attention   
*******************************************************************************/
void LPTIM_IRQHandler(void)
{
    if(FL_LPTIM32_IsEnabledIT_Update(LPTIM32) &&
       FL_LPTIM32_IsActiveFlag_Update(LPTIM32) )
    {
        LPTIM32_Tick++;
        TASK_TimeSlice(LPTIM32_Tick);

        FL_LPTIM32_ClearFlag_Update(LPTIM32);
    }
}


#if PWM_ENABLE


/*******************************************************************************
 * @brief       
 * @param       
 * @retval      
 * @attention   
*******************************************************************************/
void LPTIM16_Init(void)
{
    FL_GPIO_InitTypeDef       GPIO_InitStruct;
    FL_LPTIM16_InitTypeDef    LPTIM16_InitStruct;
    FL_LPTIM16_OC_InitTypeDef LPTIM16_OC_InitStruct;

    FL_LPTIM16_StructInit(&LPTIM16_InitStruct);
    LPTIM16_InitStruct.clockSource          = FL_CMU_LPTIM16_CLK_SOURCE_APBCLK;
    LPTIM16_InitStruct.mode                 = FL_LPTIM16_OPERATION_MODE_NORMAL;
    LPTIM16_InitStruct.prescalerClockSource = FL_LPTIM16_CLK_SOURCE_INTERNAL;
    LPTIM16_InitStruct.prescaler            = FL_LPTIM16_PSC_DIV8;
    LPTIM16_InitStruct.autoReload           = 1000 - 1;
    LPTIM16_InitStruct.encoderMode          = FL_LPTIM16_ENCODER_MODE_DISABLE;
    LPTIM16_InitStruct.onePulseMode         = FL_LPTIM16_ONE_PULSE_MODE_CONTINUOUS;
    LPTIM16_InitStruct.triggerEdge          = FL_LPTIM16_ETR_TRIGGER_EDGE_RISING;
    LPTIM16_InitStruct.countEdge            = FL_LPTIM16_ETR_COUNT_EDGE_RISING;
    FL_LPTIM16_Init(LPTIM16, &LPTIM16_InitStruct);

    FL_GPIO_StructInit(&GPIO_InitStruct);
    GPIO_InitStruct.pin        = FL_GPIO_PIN_13;
    GPIO_InitStruct.mode       = FL_GPIO_MODE_DIGITAL;
    GPIO_InitStruct.outputType = FL_GPIO_OUTPUT_PUSHPULL;
    GPIO_InitStruct.pull       = FL_DISABLE;
    GPIO_InitStruct.remapPin   = FL_ENABLE;
    FL_GPIO_Init(GPIOC, &GPIO_InitStruct);

    LPTIM16_OC_InitStruct.OCPolarity   = FL_LPTIM16_OC_POLARITY_NORMAL;
    LPTIM16_OC_InitStruct.compareValue = 500 - 1;
    FL_LPTIM16_OC_Init(LPTIM16, FL_LPTIM16_CHANNEL_1, &LPTIM16_OC_InitStruct);

    FL_LPTIM16_Enable(LPTIM16);
}


#endif


/******************* (C) COPYRIGHT 2021 *************************END OF FILE***/

