/*******************************************************************************
 * @file    OLED1.c
 * @author  King
 * @version V1.00
 * @date    27-Nov-2021
 * @brief   ......
*******************************************************************************/


/* Define to prevent recursive inclusion -------------------------------------*/
#define __OLED1_C__


/* Includes ------------------------------------------------------------------*/
#include "OLED1.h"
#include "IMAGE.h"


#if OLED1_ENABLE


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/


/* Exported variables --------------------------------------------------------*/
/* Exported function prototypes ----------------------------------------------*/


/*******************************************************************************
 * @brief       
 * @param       
 * @retval      
 * @attention   
*******************************************************************************/
void OLED1_InitI2C(void)
{
    FL_GPIO_InitTypeDef           GPIO_InitStruct;
    FL_I2C_MasterMode_InitTypeDef I2C_InitStruct;

    FL_GPIO_StructInit(&GPIO_InitStruct);
    GPIO_InitStruct.pin        = FL_GPIO_PIN_11 | FL_GPIO_PIN_12;
    GPIO_InitStruct.mode       = FL_GPIO_MODE_DIGITAL;
    GPIO_InitStruct.outputType = FL_GPIO_OUTPUT_OPENDRAIN;
    GPIO_InitStruct.pull       = FL_DISABLE;
    FL_GPIO_Init(GPIOA, &GPIO_InitStruct);

    I2C_InitStruct.clockSource = FL_CMU_I2C_CLK_SOURCE_APBCLK;
    I2C_InitStruct.baudRate    = 400000;
    FL_I2C_MasterMode_Init(I2C, &I2C_InitStruct);
}


/*******************************************************************************
 * @brief       
 * @param       
 * @retval      
 * @attention   
*******************************************************************************/
uint8_t I2C_SendData(uint8_t Data)
{
    FL_I2C_Master_WriteTXBuff(I2C, Data);
    while(!FL_I2C_Master_IsActiveFlag_TXComplete(I2C));

    FL_I2C_Master_ClearFlag_TXComplete(I2C);

    if(!FL_I2C_Master_IsActiveFlag_NACK(I2C))
    {
        return 0;
    }
    else
    {
        FL_I2C_Master_ClearFlag_NACK(I2C);  return 1;
    }
}


/*******************************************************************************
 * @brief       
 * @param       
 * @retval      
 * @attention   
*******************************************************************************/
void OLED1_I2C_WriteData(uint8_t Address, uint8_t Data)
{
    FL_I2C_Master_EnableI2CStart(I2C);
    while(!FL_I2C_Master_IsActiveFlag_Start(I2C));

    FL_I2C_Master_DisableRX(I2C);

    I2C_SendData(0x78);
    I2C_SendData(Address);
    I2C_SendData(Data);

    FL_I2C_Master_EnableI2CStop(I2C);
    while(!FL_I2C_Master_IsActiveFlag_Stop(I2C));
}


/*******************************************************************************
 * @brief       
 * @param       
 * @retval      
 * @attention   
*******************************************************************************/
void OLED1_I2C_WriteBuffer(uint8_t Address, uint8_t *Buffer, uint8_t Length)
{
    FL_I2C_Master_EnableI2CStart(I2C);
    while(!FL_I2C_Master_IsActiveFlag_Start(I2C));

    FL_I2C_Master_DisableRX(I2C);

    I2C_SendData(0x78);
    I2C_SendData(Address);

    for(uint8_t i = 0; i < Length; i++)
    {
        I2C_SendData(*Buffer++);
    }

    FL_I2C_Master_EnableI2CStop(I2C);
    while(!FL_I2C_Master_IsActiveFlag_Stop(I2C));
}


/*******************************************************************************
 * @brief       
 * @param       
 * @retval      
 * @attention   
*******************************************************************************/
void OLED1_WriteCMD(uint8_t Command)
{
    OLED1_I2C_WriteData(0x00, Command);
}


/*******************************************************************************
 * @brief       
 * @param       
 * @retval      
 * @attention   
*******************************************************************************/
void OLED1_WriteDAT(uint8_t Data)
{
    OLED1_I2C_WriteData(0x40, Data);
}

/*******************************************************************************
 * @brief       
 * @param       
 * @retval      
 * @attention   
*******************************************************************************/
void OLED1_WriteBuffer(uint8_t *Buffer, uint8_t Length)
{
    OLED1_I2C_WriteBuffer(0x40, Buffer, Length);
}


/*******************************************************************************
 * @brief       
 * @param       
 * @retval      
 * @attention   
*******************************************************************************/
void OLED1_InitCFG(void)
{
    FL_DelayMs(500);        /* ʱҪ,ϵʱûд */

    OLED1_WriteCMD(0xAE);    /* Display Off */
    OLED1_WriteCMD(0x20);    /* Set Memory Addressing Mode */
    OLED1_WriteCMD(0x10);    /* 00  Horizontal Addressing Mode
                               01  Vertical Addressing Mode
                               10  Page Addressing Mode (RESET)
                               11  Invalid */
    OLED1_WriteCMD(0xB0);    /* Set Page Start Address for Page Addressing Mode,0-7 */
    OLED1_WriteCMD(0xC8);    /* Set COM Output Scan Direction */
    OLED1_WriteCMD(0x00);    /* Set Low  Column Address */
    OLED1_WriteCMD(0x10);    /* Set High Column Address */
    OLED1_WriteCMD(0x40);    /* Set Start Line Address */
    OLED1_WriteCMD(0x81);    /* Set Contrast Control Register */
    OLED1_WriteCMD(0xFF);    /* Brightness 0x00~0xff */
    OLED1_WriteCMD(0xA1);    /* Set Segment Re-map 0 to 127 */
    OLED1_WriteCMD(0xA6);    /* Set Normal Display */
    OLED1_WriteCMD(0xA8);    /* Set Multiplex Ratio(1 to 64) */
    OLED1_WriteCMD(0x3F);
    OLED1_WriteCMD(0xA4);    /* 0xA4: Output Follows RAM Content
                               0xA5: Output Ignores RAM Content */
    OLED1_WriteCMD(0xD3);    /* Aet Display offset */
    OLED1_WriteCMD(0x00);    /* Not Offset */
    OLED1_WriteCMD(0xD5);    /* Set Display Clock Divide Ratio/Oscillator Frequency */
    OLED1_WriteCMD(0xF0);    /* Set Divide Ratio */
    OLED1_WriteCMD(0xD9);    /* Set Pre-charge Period */
    OLED1_WriteCMD(0x22);
    OLED1_WriteCMD(0xDA);    /* Set Com Pins Hardware Configuration */
    OLED1_WriteCMD(0x12);
    OLED1_WriteCMD(0xDB);    /* Set VCOMH */
    OLED1_WriteCMD(0x20);    /* 0x20=0.77*VCC */
    OLED1_WriteCMD(0x8D);    /* Set DC-DC Enable */
    OLED1_WriteCMD(0x14);
    OLED1_WriteCMD(0xAF);    /* Turn On OLED1 Panel */
}


/*******************************************************************************
 * @brief       
 * @param       
 * @retval      
 * @attention   
*******************************************************************************/
void OLED1_SetPosition(uint8_t x, uint8_t y)
{
	OLED1_WriteCMD(0xB0 + y);
	OLED1_WriteCMD(((x & 0xF0) >> 4) | 0x10);
	OLED1_WriteCMD(((x & 0x0F) >> 0) | 0x00);
}


/*******************************************************************************
 * @brief       
 * @param       
 * @retval      
 * @attention   
*******************************************************************************/
void OLED1_Clear(uint8_t Data)
{
    uint8_t x = 0, y = 0;

    for(y = 0; y < 8; y++)
    {
        OLED1_SetPosition(0, y);

        for(x = 0; x < 128; x++)
        {
            OLED1_WriteDAT(Data);
        }
    }
}


/*******************************************************************************
 * @brief       
 * @param       
 * @retval      
 * @attention   
*******************************************************************************/
void OLED1_DrawImage(uint8_t Width, uint8_t Height, const uint8_t *ImageData)
{
    uint8_t  x = 0, y = 0;
    uint8_t  Buffer[0x80];
    uint16_t Index    = 0;

    for(y = 0; y < Height / 8; y++)
    {
        OLED1_SetPosition(32, y);

        for(x = 0; x < Width; x++)
        {
            Buffer[x] = ImageData[Index++];
        }

        OLED1_WriteBuffer(Buffer, Width);
    }
}


/*******************************************************************************
 * @brief       
 * @param       
 * @retval      
 * @attention   
*******************************************************************************/
void OLED1_Init(void)
{
    OLED1_InitI2C();

    OLED1_InitCFG();

    OLED1_Clear(0x00);

    TASK_Append(TASK_ID_OLED1, OLED1_Handler, 100);
}


/*******************************************************************************
 * @brief       
 * @param       
 * @retval      
 * @attention   
*******************************************************************************/
void OLED1_Handler(void)
{
    static uint8_t Index = 0;

    switch(Index)
    {
        case   0: OLED1_DrawImage(64, 64, gImage_Clock001); break;
        case   1: OLED1_DrawImage(64, 64, gImage_Clock002); break;
        case   2: OLED1_DrawImage(64, 64, gImage_Clock003); break;
        case   3: OLED1_DrawImage(64, 64, gImage_Clock004); break;
        case   4: OLED1_DrawImage(64, 64, gImage_Clock005); break;
        case   5: OLED1_DrawImage(64, 64, gImage_Clock006); break;
        case   6: OLED1_DrawImage(64, 64, gImage_Clock007); break;
        case   7: OLED1_DrawImage(64, 64, gImage_Clock008); break;
        case   8: OLED1_DrawImage(64, 64, gImage_Clock009); break;
        case   9: OLED1_DrawImage(64, 64, gImage_Clock010); break;

        case  10: OLED1_DrawImage(64, 64, gImage_Clock011); break;
        case  11: OLED1_DrawImage(64, 64, gImage_Clock012); break;
        case  12: OLED1_DrawImage(64, 64, gImage_Clock013); break;
        case  13: OLED1_DrawImage(64, 64, gImage_Clock014); break;
        case  14: OLED1_DrawImage(64, 64, gImage_Clock015); break;
        case  15: OLED1_DrawImage(64, 64, gImage_Clock016); break;
        case  16: OLED1_DrawImage(64, 64, gImage_Clock017); break;
        case  17: OLED1_DrawImage(64, 64, gImage_Clock018); break;
        case  18: OLED1_DrawImage(64, 64, gImage_Clock019); break;
        case  19: OLED1_DrawImage(64, 64, gImage_Clock020); break;

        case  20: OLED1_DrawImage(64, 64, gImage_Clock021); break;
        case  21: OLED1_DrawImage(64, 64, gImage_Clock022); break;
        case  22: OLED1_DrawImage(64, 64, gImage_Clock023); break;
        case  23: OLED1_DrawImage(64, 64, gImage_Clock024); break;
        case  24: OLED1_DrawImage(64, 64, gImage_Clock025); break;
        case  25: OLED1_DrawImage(64, 64, gImage_Clock026); break;
        case  26: OLED1_DrawImage(64, 64, gImage_Clock027); break;
        case  27: OLED1_DrawImage(64, 64, gImage_Clock028); break;
        case  28: OLED1_DrawImage(64, 64, gImage_Clock029); break;
        case  29: OLED1_DrawImage(64, 64, gImage_Clock030); break;

        case  30: OLED1_DrawImage(64, 64, gImage_Clock031); break;
        case  31: OLED1_DrawImage(64, 64, gImage_Clock032); break;
        case  32: OLED1_DrawImage(64, 64, gImage_Clock033); break;
        case  33: OLED1_DrawImage(64, 64, gImage_Clock034); break;
        case  34: OLED1_DrawImage(64, 64, gImage_Clock035); break;
        case  35: OLED1_DrawImage(64, 64, gImage_Clock036); break;
        case  36: OLED1_DrawImage(64, 64, gImage_Clock037); break;
        case  37: OLED1_DrawImage(64, 64, gImage_Clock038); break;
        case  38: OLED1_DrawImage(64, 64, gImage_Clock039); break;
        case  39: OLED1_DrawImage(64, 64, gImage_Clock040); break;

        case  40: OLED1_DrawImage(64, 64, gImage_Clock041); break;
        case  41: OLED1_DrawImage(64, 64, gImage_Clock042); break;
        case  42: OLED1_DrawImage(64, 64, gImage_Clock043); break;
        case  43: OLED1_DrawImage(64, 64, gImage_Clock044); break;
        case  44: OLED1_DrawImage(64, 64, gImage_Clock045); break;
        case  45: OLED1_DrawImage(64, 64, gImage_Clock046); break;
        case  46: OLED1_DrawImage(64, 64, gImage_Clock047); break;
        case  47: OLED1_DrawImage(64, 64, gImage_Clock048); break;
        case  48: OLED1_DrawImage(64, 64, gImage_Clock049); break;
        case  49: OLED1_DrawImage(64, 64, gImage_Clock050); break;

        case  50: OLED1_DrawImage(64, 64, gImage_Clock051); break;
        case  51: OLED1_DrawImage(64, 64, gImage_Clock052); break;
        case  52: OLED1_DrawImage(64, 64, gImage_Clock053); break;
        case  53: OLED1_DrawImage(64, 64, gImage_Clock054); break;
        case  54: OLED1_DrawImage(64, 64, gImage_Clock055); break;
        case  55: OLED1_DrawImage(64, 64, gImage_Clock056); break;
        case  56: OLED1_DrawImage(64, 64, gImage_Clock057); break;
        case  57: OLED1_DrawImage(64, 64, gImage_Clock058); break;
        case  58: OLED1_DrawImage(64, 64, gImage_Clock059); break;
        case  59: OLED1_DrawImage(64, 64, gImage_Clock060); break;

        case  60: OLED1_DrawImage(64, 64, gImage_Clock061); break;
        case  61: OLED1_DrawImage(64, 64, gImage_Clock062); break;
        case  62: OLED1_DrawImage(64, 64, gImage_Clock063); break;
        case  63: OLED1_DrawImage(64, 64, gImage_Clock064); break;
        case  64: OLED1_DrawImage(64, 64, gImage_Clock065); break;
        case  65: OLED1_DrawImage(64, 64, gImage_Clock066); break;
        case  66: OLED1_DrawImage(64, 64, gImage_Clock067); break;
        case  67: OLED1_DrawImage(64, 64, gImage_Clock068); break;
        case  68: OLED1_DrawImage(64, 64, gImage_Clock069); break;
        case  69: OLED1_DrawImage(64, 64, gImage_Clock070); break;

        case  70: OLED1_DrawImage(64, 64, gImage_Clock071); break;
        case  71: OLED1_DrawImage(64, 64, gImage_Clock072); break;
        case  72: OLED1_DrawImage(64, 64, gImage_Clock073); break;
        case  73: OLED1_DrawImage(64, 64, gImage_Clock074); break;
        case  74: OLED1_DrawImage(64, 64, gImage_Clock075); break;
        case  75: OLED1_DrawImage(64, 64, gImage_Clock076); break;
        case  76: OLED1_DrawImage(64, 64, gImage_Clock077); break;
        case  77: OLED1_DrawImage(64, 64, gImage_Clock078); break;
        case  78: OLED1_DrawImage(64, 64, gImage_Clock079); break;
        case  79: OLED1_DrawImage(64, 64, gImage_Clock080); break;

        case  80: OLED1_DrawImage(64, 64, gImage_Clock081); break;
        case  81: OLED1_DrawImage(64, 64, gImage_Clock082); break;
        case  82: OLED1_DrawImage(64, 64, gImage_Clock083); break;
        case  83: OLED1_DrawImage(64, 64, gImage_Clock084); break;
        case  84: OLED1_DrawImage(64, 64, gImage_Clock085); break;
        case  85: OLED1_DrawImage(64, 64, gImage_Clock086); break;
        case  86: OLED1_DrawImage(64, 64, gImage_Clock087); break;
        case  87: OLED1_DrawImage(64, 64, gImage_Clock088); break;
        case  88: OLED1_DrawImage(64, 64, gImage_Clock089); break;
        case  89: OLED1_DrawImage(64, 64, gImage_Clock090); break;

        case  90: OLED1_DrawImage(64, 64, gImage_Clock091); break;
        case  91: OLED1_DrawImage(64, 64, gImage_Clock092); break;
        case  92: OLED1_DrawImage(64, 64, gImage_Clock093); break;
        case  93: OLED1_DrawImage(64, 64, gImage_Clock094); break;
        case  94: OLED1_DrawImage(64, 64, gImage_Clock095); break;
        case  95: OLED1_DrawImage(64, 64, gImage_Clock096); break;
        case  96: OLED1_DrawImage(64, 64, gImage_Clock097); break;
        case  97: OLED1_DrawImage(64, 64, gImage_Clock098); break;
        case  98: OLED1_DrawImage(64, 64, gImage_Clock099); break;
        case  99: OLED1_DrawImage(64, 64, gImage_Clock100); break;

        case 100: OLED1_DrawImage(64, 64, gImage_Clock101); break;
        case 101: OLED1_DrawImage(64, 64, gImage_Clock102); break;
        case 102: OLED1_DrawImage(64, 64, gImage_Clock103); break;
        case 103: OLED1_DrawImage(64, 64, gImage_Clock104); break;
        case 104: OLED1_DrawImage(64, 64, gImage_Clock105); break;
        case 105: OLED1_DrawImage(64, 64, gImage_Clock106); break;
        case 106: OLED1_DrawImage(64, 64, gImage_Clock107); break;
        case 107: OLED1_DrawImage(64, 64, gImage_Clock108); break;
        case 108: OLED1_DrawImage(64, 64, gImage_Clock109); break;
        case 109: OLED1_DrawImage(64, 64, gImage_Clock110); break;

        case 110: OLED1_DrawImage(64, 64, gImage_Clock111); break;
        case 111: OLED1_DrawImage(64, 64, gImage_Clock112); break;
        case 112: OLED1_DrawImage(64, 64, gImage_Clock113); break;
        case 113: OLED1_DrawImage(64, 64, gImage_Clock114); break;
        case 114: OLED1_DrawImage(64, 64, gImage_Clock115); break;
        case 115: OLED1_DrawImage(64, 64, gImage_Clock116); break;
        case 116: OLED1_DrawImage(64, 64, gImage_Clock117); break;
        case 117: OLED1_DrawImage(64, 64, gImage_Clock118); break;
        case 118: OLED1_DrawImage(64, 64, gImage_Clock119); break;
        case 119: OLED1_DrawImage(64, 64, gImage_Clock120); break;

        case 120: OLED1_DrawImage(64, 64, gImage_Clock121); break;
        case 121: OLED1_DrawImage(64, 64, gImage_Clock122); break;
        case 122: OLED1_DrawImage(64, 64, gImage_Clock123); break;
        case 123: OLED1_DrawImage(64, 64, gImage_Clock124); break;
        case 124: OLED1_DrawImage(64, 64, gImage_Clock125); break;
        case 125: OLED1_DrawImage(64, 64, gImage_Clock126); break;
        case 126: OLED1_DrawImage(64, 64, gImage_Clock127); break;
        case 127: OLED1_DrawImage(64, 64, gImage_Clock128); break;
        case 128: OLED1_DrawImage(64, 64, gImage_Clock129); break;
        case 129: OLED1_DrawImage(64, 64, gImage_Clock130); break;

        case 130: OLED1_DrawImage(64, 64, gImage_Clock131); break;
        case 131: OLED1_DrawImage(64, 64, gImage_Clock132); break;
        case 132: OLED1_DrawImage(64, 64, gImage_Clock133); break;
        case 133: OLED1_DrawImage(64, 64, gImage_Clock134); break;
        case 134: OLED1_DrawImage(64, 64, gImage_Clock135); break;
        case 135: OLED1_DrawImage(64, 64, gImage_Clock136); break;
        case 136: OLED1_DrawImage(64, 64, gImage_Clock137); break;
        case 137: OLED1_DrawImage(64, 64, gImage_Clock138); break;
        case 138: OLED1_DrawImage(64, 64, gImage_Clock139); break;
        case 139: OLED1_DrawImage(64, 64, gImage_Clock140); break;

        case 140: OLED1_DrawImage(64, 64, gImage_Clock141); break;
        case 141: OLED1_DrawImage(64, 64, gImage_Clock142); break;
        case 142: OLED1_DrawImage(64, 64, gImage_Clock143); break;
        default: break;
    }

    Index = (Index + 1) % 143;
}


#endif


/******************* (C) COPYRIGHT 2021 *************************END OF FILE***/

