/*******************************************************************************
 * @file    OLED2.c
 * @author  King
 * @version V1.00
 * @date    27-Nov-2021
 * @brief   ......
*******************************************************************************/


/* Define to prevent recursive inclusion -------------------------------------*/
#define __OLED2_C__


/* Includes ------------------------------------------------------------------*/
#include "OLED2.h"
#include "FONT.h"


#if OLED2_ENABLE


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/


/* Private macro -------------------------------------------------------------*/
#define OLED2_DC_H()    FL_GPIO_SetOutputPin(  GPIOC, FL_GPIO_PIN_0)
#define OLED2_DC_L()    FL_GPIO_ResetOutputPin(GPIOC, FL_GPIO_PIN_0)

#define OLED2_RST_H()   FL_GPIO_SetOutputPin(  GPIOC, FL_GPIO_PIN_1)
#define OLED2_RST_L()   FL_GPIO_ResetOutputPin(GPIOC, FL_GPIO_PIN_1)

#define OLED2_CS_H()    FL_SPI_SetSSNPin(SPI1, FL_SPI_SSN_HIGH)
#define OLED2_CS_L()    FL_SPI_SetSSNPin(SPI1, FL_SPI_SSN_LOW)


/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/


/* Exported variables --------------------------------------------------------*/
/* Exported function prototypes ----------------------------------------------*/


/*******************************************************************************
 * @brief       
 * @param       
 * @retval      
 * @attention   
*******************************************************************************/
void OLED2_InitSPI(void)
{
    FL_DMA_InitTypeDef  DMA_InitStruct;
    FL_GPIO_InitTypeDef GPIO_InitStruct;
    FL_SPI_InitTypeDef  SPI_InitStruct;

    FL_SPI_StructInit(&SPI_InitStruct);
    SPI_InitStruct.baudRate      = FL_SPI_BAUDRATE_DIV2;
    SPI_InitStruct.bitOrder      = FL_SPI_BIT_ORDER_MSB_FIRST;
    SPI_InitStruct.clockPhase    = FL_SPI_PHASE_EDGE1;
    SPI_InitStruct.clockPolarity = FL_SPI_POLARITY_NORMAL;
    SPI_InitStruct.dataWidth     = FL_SPI_DATA_WIDTH_8B;
    SPI_InitStruct.mode          = FL_SPI_WORK_MODE_MASTER;
    SPI_InitStruct.softControl   = FL_ENABLE;
    SPI_InitStruct.transferMode  = FL_SPI_TRANSFER_MODE_FULL_DUPLEX;
    FL_SPI_Init(SPI1, &SPI_InitStruct);

    FL_DMA_StructInit(&DMA_InitStruct);
    DMA_InitStruct.circMode = FL_DISABLE;
    DMA_InitStruct.dataSize = FL_DMA_BANDWIDTH_8B;
    DMA_InitStruct.direction = FL_DMA_DIR_PERIPHERAL_TO_RAM;
    DMA_InitStruct.memoryAddressIncMode = FL_DMA_MEMORY_INC_MODE_INCREASE;
    DMA_InitStruct.priority = FL_DMA_PRIORITY_HIGH ;
    DMA_InitStruct.periphAddress = FL_DMA_PERIPHERAL_FUNCTION1;
    FL_DMA_Init(DMA, &DMA_InitStruct, FL_DMA_CHANNEL_3);

    FL_DMA_StructInit(&DMA_InitStruct);
    DMA_InitStruct.circMode = FL_DISABLE;
    DMA_InitStruct.dataSize = FL_DMA_BANDWIDTH_8B;
    DMA_InitStruct.direction = FL_DMA_DIR_RAM_TO_PERIPHERAL;
    DMA_InitStruct.memoryAddressIncMode = FL_DMA_MEMORY_INC_MODE_INCREASE;
    DMA_InitStruct.priority = FL_DMA_PRIORITY_HIGH ;
    DMA_InitStruct.periphAddress = FL_DMA_PERIPHERAL_FUNCTION2;
    FL_DMA_Init(DMA, &DMA_InitStruct, FL_DMA_CHANNEL_4);

    FL_GPIO_StructInit(&GPIO_InitStruct);
    GPIO_InitStruct.pin        = FL_GPIO_PIN_2 | FL_GPIO_PIN_3 |
                                 FL_GPIO_PIN_4 |FL_GPIO_PIN_5;
    GPIO_InitStruct.mode       = FL_GPIO_MODE_DIGITAL;
    GPIO_InitStruct.outputType = FL_GPIO_OUTPUT_PUSHPULL;
    FL_GPIO_Init(GPIOD, &GPIO_InitStruct);
}


/*******************************************************************************
 * @brief       
 * @param       
 * @retval      
 * @attention   
*******************************************************************************/
void OLED2_SPI_WriteByte(uint8_t Data)
{
    FL_SPI_WriteTXBuff(SPI1, Data);
    while(!(FL_SPI_IsActiveFlag_TXBuffEmpty(SPI1)));
}


/*******************************************************************************
 * @brief       
 * @param       
 * @retval      
 * @attention   
*******************************************************************************/
void OLED2_WriteCMD(uint8_t Command)
{
    OLED2_DC_L();
    OLED2_CS_L();
    OLED2_SPI_WriteByte(Command);
    OLED2_CS_H();
}


/*******************************************************************************
 * @brief       
 * @param       
 * @retval      
 * @attention   
*******************************************************************************/
void OLED2_WriteDAT(uint8_t Data)
{
    OLED2_DC_H();
    OLED2_CS_L();
    OLED2_SPI_WriteByte(Data);
    OLED2_CS_H();
}


/*******************************************************************************
 * @brief       
 * @param       
 * @retval      
 * @attention   
*******************************************************************************/
void OLED2_ConfigureREG(void)
{
	OLED2_WriteCMD(0xAE);    //--turn off oled panel
	OLED2_WriteCMD(0x00);    //---set low column address
	OLED2_WriteCMD(0x10);    //---set high column address
	OLED2_WriteCMD(0x40);    //--set start line address  Set Mapping RAM Display Start Line (0x00~0x3F)
	OLED2_WriteCMD(0x81);    //--set contrast control register
	OLED2_WriteCMD(0xCF);    // Set SEG Output Current Brightness
	OLED2_WriteCMD(0xA1);    //--Set SEG/Column Mapping     0xa0ҷ 0xa1
	OLED2_WriteCMD(0xC0);    //Set COM/Row Scan Direction   0xc0· 0xc8
	OLED2_WriteCMD(0xA6);    //--set normal display
	OLED2_WriteCMD(0xA8);    //--set multiplex ratio(1 to 64)
	OLED2_WriteCMD(0x3f);    //--1/64 duty
	OLED2_WriteCMD(0xD3);    //-set display offset	Shift Mapping RAM Counter (0x00~0x3F)
	OLED2_WriteCMD(0x00);    //-not offset
	OLED2_WriteCMD(0xd5);    //--set display clock divide ratio/oscillator frequency
	OLED2_WriteCMD(0x80);    //--set divide ratio, Set Clock as 100 Frames/Sec
	OLED2_WriteCMD(0xD9);    //--set pre-charge period
	OLED2_WriteCMD(0xF1);    //Set Pre-Charge as 15 Clocks & Discharge as 1 Clock
	OLED2_WriteCMD(0xDA);    //--set com pins hardware configuration
	OLED2_WriteCMD(0x12);    
	OLED2_WriteCMD(0xDB);    //--set vcomh
	OLED2_WriteCMD(0x40);    //Set VCOM Deselect Level
	OLED2_WriteCMD(0x20);    //-Set Page Addressing Mode (0x00/0x01/0x02)
	OLED2_WriteCMD(0x02);    //
	OLED2_WriteCMD(0x8D);    //--set Charge Pump enable/disable
	OLED2_WriteCMD(0x14);    //--set(0x10) disable
	OLED2_WriteCMD(0xA4);    // Disable Entire Display On (0xa4/0xa5)
	OLED2_WriteCMD(0xA6);    // Disable Inverse Display On (0xa6/a7) 
	OLED2_WriteCMD(0xAF);    //--turn on oled panel
	OLED2_WriteCMD(0xAF);    //---display ON
}


/*******************************************************************************
 * @brief       
 * @param       
 * @retval      
 * @attention   
*******************************************************************************/
void OLED2_SetPosition(uint8_t x, uint8_t y)
{
	OLED2_WriteCMD(0xB0 + y);
	OLED2_WriteCMD(((x & 0xF0) >> 4) | 0x10);
	OLED2_WriteCMD(((x & 0x0F) >> 0) | 0x00);
}


/*******************************************************************************
 * @brief       
 * @param       
 * @retval      
 * @attention   
*******************************************************************************/
void OLED2_Clear(uint8_t Data)
{
    uint8_t x = 0, y = 0;

    for(y = 0; y < 8; y++)
    {
        OLED2_SetPosition(0, y);

        for(x = 0; x < 128; x++)
        {
            OLED2_WriteDAT(Data);
        }
    }
}


/*******************************************************************************
 * @brief       
 * @param       
 * @retval      
 * @attention   
*******************************************************************************/
void OLED2_HardwareReset(void)
{
    FL_GPIO_InitTypeDef GPIO_InitStruct;

    FL_GPIO_StructInit(&GPIO_InitStruct);
    GPIO_InitStruct.pin        = FL_GPIO_PIN_0 | FL_GPIO_PIN_1;
    GPIO_InitStruct.mode       = FL_GPIO_MODE_OUTPUT;
    GPIO_InitStruct.outputType = FL_GPIO_OUTPUT_PUSHPULL;
    FL_GPIO_Init(GPIOC, &GPIO_InitStruct);

    OLED2_RST_H(); FL_DelayMs(100);
    OLED2_RST_L(); FL_DelayMs(100);
    OLED2_RST_H(); FL_DelayMs(100);
}


/*******************************************************************************
 * @brief       
 * @param       
 * @retval      
 * @attention   
*******************************************************************************/
void OLED2_Init(void)
{
    OLED2_InitSPI();

    OLED2_HardwareReset();

    OLED2_ConfigureREG();

    OLED2_Clear(0x00);

    TASK_Append(TASK_ID_OLED2, OLED2_Handler, 500);
}


/*******************************************************************************
 * @brief       
 * @param       
 * @retval      
 * @attention   
*******************************************************************************/
void OLED_ShowChar(uint8_t X, uint8_t Y, char ch, uint8_t Size)
{  
    uint8_t Data = 0;

    for(uint8_t i = 0; i < Size; i++)
    {
        OLED2_SetPosition(X + i, Y);

        if(Size == 12)
        {
            Data = ASCII_1206[ch - 0x20][i];
        }
        else
        {
            Data = ASCII_1608[ch - 0x20][i];
        }

        OLED2_WriteDAT(Data);
    }
}


/*******************************************************************************
 * @brief       
 * @param       
 * @retval      
 * @attention   
*******************************************************************************/
void OLED2_Handler(void)
{
    char Buffer[20];

    FL_RTCB_InitTypeDef    RTCB_InitStruct;
    FL_RTCB_GetTime(RTCB, &RTCB_InitStruct);


    memset(Buffer, 0, sizeof(Buffer));
    sprintf(Buffer, "20%02x",RTCB_InitStruct.year);

    OLED_ShowChar(10, 0, 'Y',       16);
    OLED_ShowChar(10, 1, ':',       16);
    OLED_ShowChar(10, 2, Buffer[0], 16);
    OLED_ShowChar(10, 3, Buffer[1], 16);
    OLED_ShowChar(10, 4, Buffer[2], 16);
    OLED_ShowChar(10, 5, Buffer[3], 16);
    

    memset(Buffer, 0, sizeof(Buffer));
    sprintf(Buffer, "%02x", RTCB_InitStruct.month);

    OLED_ShowChar(26, 0, 'M',       16);
    OLED_ShowChar(26, 1, ':',       16);
    OLED_ShowChar(26, 2, Buffer[0], 16);
    OLED_ShowChar(26, 3, Buffer[1], 16);


    memset(Buffer, 0, sizeof(Buffer));
    sprintf(Buffer, "%02x", RTCB_InitStruct.day);

    OLED_ShowChar(42, 0, 'D',       16);
    OLED_ShowChar(42, 1, ':',       16);
    OLED_ShowChar(42, 2, Buffer[0], 16);
    OLED_ShowChar(42, 3, Buffer[1], 16);


    switch(RTCB_InitStruct.week)
    {
        case 0 :
            OLED_ShowChar(66, 0, 'S', 16);
            OLED_ShowChar(66, 1, 'U', 16);
            OLED_ShowChar(66, 2, 'N', 16);
            break;

        case 1 :
            OLED_ShowChar(66, 0, 'M', 16);
            OLED_ShowChar(66, 1, 'O', 16);
            OLED_ShowChar(66, 2, 'N', 16);
            break;

        case 2 :
            OLED_ShowChar(66, 0, 'T', 16);
            OLED_ShowChar(66, 1, 'U', 16);
            OLED_ShowChar(66, 2, 'E', 16);
            break;

        case 3 :
            OLED_ShowChar(66, 0, 'W', 16);
            OLED_ShowChar(66, 1, 'E', 16);
            OLED_ShowChar(66, 2, 'D', 16);
            break;

        case 4 :
            OLED_ShowChar(66, 0, 'T', 16);
            OLED_ShowChar(66, 1, 'H', 16);
            OLED_ShowChar(66, 2, 'U', 16);
            break;

        case 5 :
            OLED_ShowChar(66, 0, 'F', 16);
            OLED_ShowChar(66, 1, 'R', 16);
            OLED_ShowChar(66, 2, 'I', 16);
            break;

        case 6 :
            OLED_ShowChar(66, 0, 'S', 16);
            OLED_ShowChar(66, 1, 'A', 16);
            OLED_ShowChar(66, 2, 'T', 16);
            break;

        default:break;
    }


    memset(Buffer, 0, sizeof(Buffer));
    sprintf(Buffer, "%02x:%02x:%02x",
            RTCB_InitStruct.hour, RTCB_InitStruct.minute, RTCB_InitStruct.second);

    OLED_ShowChar(90, 0, Buffer[0], 16);
    OLED_ShowChar(90, 1, Buffer[1], 16);
    OLED_ShowChar(90, 2, Buffer[2], 16);
    OLED_ShowChar(90, 3, Buffer[3], 16);
    OLED_ShowChar(90, 4, Buffer[4], 16);
    OLED_ShowChar(90, 5, Buffer[5], 16);
    OLED_ShowChar(90, 6, Buffer[6], 16);
    OLED_ShowChar(90, 7, Buffer[7], 16);
}


#endif


/******************* (C) COPYRIGHT 2021 *************************END OF FILE***/

